//=============================================================================
// Project:    SLP - An Interpreter for Super Logic Programs / Static Semantics
// Module:     rlit.cxx
// Purpose:    Literal (actually, an atomic formula) in a rule
// Last Change:11.10.2001
// Language:   C++
// Author:     Stefan Brass
// Email:      Stefan.Brass@informatik.uni-giessen.de, sbrass@sis.pitt.edu
// Address:    Universitaet Giessen, Arndtstr. 2, 35392 Giessen, Germany
// Copyright:  (c) 2001 by Stefan Brass
// Licence:    Distributed under the conditions of the GNU Public Licence.
//=============================================================================

//=============================================================================
// Interface:
//=============================================================================

#include "ver.h"
#include "out.h"
#include "check.h"

#include "rlit.h"

//=============================================================================
// Exported Functions:
//=============================================================================

//-----------------------------------------------------------------------------
// Constructor:
//-----------------------------------------------------------------------------

rlit_c::rlit_c(lit_t lit, rvars_t vars, bool_t binding)
{
	tlist_t	args;
	rarg_t rarg;
	rargs_t next, previous;

	// Check Parameters:
	CHECK(lit != 0, "rlit_c::Constructor: lit is NIL");
	CHECK(lit->valid(), "rlit_c::Constructor: lit is invalid");
	CHECK(vars != 0, "rlit_c::Constructor: vars is NIL");
	CHECK(vars->valid(), "rlit_c::Constructor: vars is invalid");
	CHECK(binding==BOOL_TRUE || binding==BOOL_FALSE,
			"rlit_c::Constructor: binding is invalid");

	// Initialize Magic Number:
#ifdef VER_DEBUG
	Magic = RLIT_MAGIC;
#endif

	// Initialize member variables:
	Pred = lit->get_pred();
	Args = 0;
	IsGround = BOOL_TRUE;
	GLit = 0;

	// Copy/Translate argument list:
	args = lit->get_args();
	previous = 0;
	while(args != 0) {
		rarg = new rarg_c(args->head(), vars, binding);
		if(rarg->is_cnst() != BOOL_TRUE)
			IsGround = BOOL_FALSE;
		next = new rargs_c(rarg);
		if(previous == 0)
			Args = next;
		else
			previous->append(next);
		previous = next;
		args = args->tail();
	}

	// Create/Retrieve fact_c object if this literal is ground:
	if(IsGround)
		GLit = glit_c::store(lit);

}

//-----------------------------------------------------------------------------
// Destructor:
//-----------------------------------------------------------------------------

rlit_c::~rlit_c(void)
{
#ifdef	VER_DEBUG
	Magic = 0;
#endif
}

//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
bool_t	rlit_c::valid(void) const
{
	// Check object pointer:
	CHECK(this != 0, "rlit_c::valid: NIL object");

	// Magic number must be correct:
	if(Magic != RLIT_MAGIC)
		return(BOOL_FALSE);

	// The predicate cannot be NIL:
	CHECK(Pred != 0, "rlit_c::valid: Pred is NIL");

	// The predicate must be valid:
	CHECK(Pred->valid(), "rlit_c::valid: Pred is invalid");

	// The argument list must be valid:
	CHECK(Args == 0 || Args->valid(), "rlit_c::valid: Args is invalid");

	// IsGround must be true or false:
	CHECK(IsGround == BOOL_TRUE || IsGround == BOOL_FALSE,
		"rlit_c::valid: IsGround is invalid");

	// If the literal is ground, GLit must be set and valid:
	CHECK(IsGround != BOOL_FALSE || GLit == 0,
		"rlit_c::valid: GLit must be NIL if literal not ground");
	CHECK(IsGround != BOOL_TRUE || GLit->valid(),
		"rlit_c::valid: GLit must be valid if literal is ground");

	// Ok:
	return(BOOL_TRUE);
}
#endif

//-----------------------------------------------------------------------------
// Print This Literal:
//-----------------------------------------------------------------------------

void rlit_c::print(rvars_t vars) const
{
	// Check Consistency:
	CHECK(valid(), "rlit_c::print: Invalid Object");

	// Print predicate:
	Pred->print();

	// Print argument list if not empty:
	if(Args != 0) {
		out_c::print_char('(');
		Args->print(vars);
		out_c::print_char(')');
	}
}

